package com.example.michael.bp_as;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.SharedPreferences;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.os.Bundle;
import android.app.ListFragment;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.ListAdapter;
import android.widget.ListView;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;


public class ArtListFragment extends ListFragment implements View.OnClickListener{

    private static final String TAG_ARTICLES = "clanky";
    private static final String TAG_ID = "id_text";
    private static final String TAG_HEADLINE = "nazev";
    private static final String TAG_PEREX = "perex";
    private static final String TAG_URL = "url";
    private static int page;
    private static boolean page_changed=false;
    private static boolean first;

    private OnFragmentInteractionListener mListener;

    public ArtListFragment() {
    }

    public static ArtListFragment newInstance(int p,boolean f) {
        ArtListFragment fragment=new ArtListFragment();
        Bundle args = new Bundle();
        fragment.setArguments(args);
        page=p;
        first=f;
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (first)
            triggerDownload("http://www.agris.cz/dalsi-novinky?vratmi=json&page=" + page + "&pageSize=20");
        else{
            ListAdapter mAdapter = new ArrayAdapter<>(getActivity(),android.R.layout.simple_list_item_1,android.R.id.text1,ArticleList.ITEMS);
            setListAdapter(mAdapter);
        }
    }


    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        LayoutInflater layoutInflater = getActivity().getLayoutInflater();
        LinearLayout listFooterView = (LinearLayout)layoutInflater.inflate(R.layout.footer,null);
        getListView().addFooterView(listFooterView);
        view.findViewById(R.id.butNext).setOnClickListener(this);
        view.findViewById(R.id.butPrev).setOnClickListener(this);
        if (page==1)
            view.findViewById(R.id.butPrev).setEnabled(false);
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.butNext:
                page++;
                page_changed=true;
                triggerDownload("http://www.agris.cz/dalsi-novinky?vratmi=json&page=" + page + "&pageSize=20");
                break;
            case R.id.butPrev:
                page--;
                page_changed=true;
                triggerDownload("http://www.agris.cz/dalsi-novinky?vratmi=json&page=" + page + "&pageSize=20");
                break;
        }
    }

    @Override
    public void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
        if (mListener != null) {
            mListener.loadArticleFragment(ArticleList.ITEMS.get(position).id, ArticleList.ITEMS.get(position));
        }
    }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);
        try {
            mListener = (OnFragmentInteractionListener) getActivity();
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString()
                    + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    public interface OnFragmentInteractionListener {
        void loadArticleFragment(String id, ArticleList.ArticleItem articleItem);
        void reloadArtListFragment(int page,boolean first);
    }


    private void triggerDownload(String stringUrl) {
        ConnectivityManager connMan = (ConnectivityManager)getActivity().getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo netInfo = connMan.getActiveNetworkInfo();
        if (netInfo!= null && netInfo.isConnected())
            new DownloadList().execute(stringUrl);
        else{
            AlertDialog.Builder alertBuilder= new AlertDialog.Builder(getActivity());
            alertBuilder.setMessage(R.string.no_connection);
            AlertDialog alertDialog = alertBuilder.create();
            alertDialog.show();
        }
    }


    private class DownloadList extends AsyncTask<String, Void, Void> {
        String jsonString;
        private ProgressDialog pDialog;

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            pDialog = new ProgressDialog(getActivity());
            pDialog.setMessage(getResources().getString(R.string.loading_data));
            pDialog.setCancelable(false);
            pDialog.show();
        }

        @Override
        protected Void doInBackground(String... urls) {
            try{
                jsonString = downloadUrl(urls[0]);
            }catch (IOException e){
                Log.e("Service Handler", "problem");
                return null;
            }
            return null;
        }

        private String downloadUrl(String myurl)throws IOException{
            InputStream inputStream = null;
            URL url = new URL(myurl);
            HttpURLConnection connection =  (HttpURLConnection) url.openConnection();
            connection.setReadTimeout(10000);
            connection.setConnectTimeout(15000);
            connection.setRequestMethod("GET");
            connection.setDoInput(true);
            try {connection.connect();
                inputStream = connection.getInputStream();
                java.util.Scanner scanner = new java.util.Scanner(inputStream).useDelimiter("\\A");
                return scanner.hasNext() ? scanner.next():"";
            }finally {
                if (inputStream!=null)
                    inputStream.close();
                connection.disconnect();
            }
        }

        @Override
        protected void onPostExecute(Void result) {
            super.onPostExecute(result);

            if (pDialog.isShowing())
                pDialog.dismiss();

            if (jsonString!=null){
                try {
                    ArticleList.clearArticlesList();
                    JSONObject jsonObject = new JSONObject(jsonString);
                    JSONArray articlesArr;
                    articlesArr = jsonObject.getJSONArray(TAG_ARTICLES);
                    for (int i=0;i<articlesArr.length();i++){
                        JSONObject articlesObj = articlesArr.getJSONObject(i);
                        String id = articlesObj.getString(TAG_ID);
                        String headline = articlesObj.getString(TAG_HEADLINE);
                        String perex = articlesObj.getString(TAG_PEREX);
                        String url =articlesObj.getString(TAG_URL);
                        ArticleList.ArticleItem articleItem=new ArticleList.ArticleItem(id, headline, perex, url);
                        ArticleList.addArticleItem(articleItem);
                    }
                }catch (JSONException e){
                    e.printStackTrace();
                }
            }else{
                Log.e("ServiceHandler", "Couldn't get any data from the url");
            }
            if (first) {
                ListAdapter mAdapter = new ArrayAdapter<>(getActivity(), android.R.layout.simple_list_item_1, android.R.id.text1, ArticleList.ITEMS);
                setListAdapter(mAdapter);
            }
            if (page_changed)
                mListener.reloadArtListFragment(page,false);
        }
    }
}
