package com.example.michael.bp_as;


import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.WebView;
import android.widget.LinearLayout;
import android.widget.TextView;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;


public class ArticleFragment extends Fragment implements View.OnClickListener{

    private static final String TAG_HEADLINE = "title";
    private static final String TAG_PEREX = "perex";
    private static final String TAG_DATE = "date";
    private static final String TAG_TEXT = "text";

    public static String idArticle;
    public static ArticleList.ArticleItem artItem;
    static Article article;

    static String title;
    static String date;
    static String perex;
    static String text;

    TextView tvTitle;
    TextView tvDate;
    TextView tvPerex;
    WebView webview;
    LinearLayout urlLayout;

    public ArticleFragment() {
    }

    public static ArticleFragment newInstance(String id, ArticleList.ArticleItem articleItem) {
        ArticleFragment fragment = new ArticleFragment();
        Bundle args = new Bundle();
        fragment.setArguments(args);
        idArticle=id;
        artItem=articleItem;
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments()!=null) {
            if (artItem.url.equals(""))
                triggerDownload("http://www.agris.cz/clanek/" + (idArticle) + "?vratmi=json");
            else
                article = new Article(artItem.headline, idArticle, null, artItem.perex, null, artItem.url);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_article, container, false);
    }


    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        view.findViewById(R.id.openUrl).setOnClickListener(this);

        tvTitle=(TextView)getActivity().findViewById(R.id.aTitle);
        tvDate=(TextView)getActivity().findViewById(R.id.aDate);
        tvPerex=(TextView)getActivity().findViewById(R.id.aPerex);
        webview=(WebView)getActivity().findViewById(R.id.aText);
        urlLayout=(LinearLayout)getActivity().findViewById(R.id.urlLayout);

        if (getArguments()!=null) {
            if (artItem.url.equals("")) {
            } else {
                urlLayout.setVisibility(View.VISIBLE);
                tvDate.setVisibility(View.GONE);
                webview.setVisibility(View.GONE);
                tvTitle.setText(article.title);
                tvPerex.setText(article.perex);
            }
        }
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.openUrl:
                Intent browserIntent = new Intent(Intent.ACTION_VIEW, Uri.parse(artItem.url));
                startActivity(browserIntent);
                break;
        }
    }

    private void triggerDownload(String stringUrl) {
        ConnectivityManager connMan = (ConnectivityManager)getActivity().getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo netInfo = connMan.getActiveNetworkInfo();
        if (netInfo!= null && netInfo.isConnected())
            new DownloadArticle().execute(stringUrl);
        else{
            AlertDialog.Builder alertBuilder= new AlertDialog.Builder(getActivity());
            alertBuilder.setMessage(R.string.no_connection);
            AlertDialog alertDialog = alertBuilder.create();
            alertDialog.show();
        }
    }

    private class DownloadArticle extends AsyncTask<String, Void, Void> {
        String jsonString;
        private ProgressDialog pDialog;

        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            pDialog = new ProgressDialog(getActivity());
            pDialog.setMessage(getResources().getString(R.string.loading_data));
            pDialog.setCancelable(false);
            pDialog.show();
        }

        @Override
        protected Void doInBackground(String... urls) {
            try{
                jsonString = downloadUrl(urls[0]);
            }catch (IOException e){
                Log.e("Service Handler", "problem");
                return null;
            }
            return null;
        }

        private String downloadUrl(String myurl)throws IOException{
            InputStream inputStream = null;
            URL url = new URL(myurl);
            HttpURLConnection connection =  (HttpURLConnection) url.openConnection();
            connection.setReadTimeout(10000);
            connection.setConnectTimeout(15000);
            connection.setRequestMethod("GET");
            connection.setDoInput(true);
            try {
                connection.connect();
                inputStream = connection.getInputStream();

                java.util.Scanner scanner = new java.util.Scanner(inputStream).useDelimiter("\\A");
                return scanner.hasNext() ? scanner.next():"";
            }finally {
                if (inputStream!=null)
                    inputStream.close();
                connection.disconnect();
            }
        }

        @Override
        protected void onPostExecute(Void aVoid) {
            super.onPostExecute(aVoid);

            if (pDialog.isShowing())
                pDialog.dismiss();

            if (jsonString != null) {
                try {
                    JSONObject aJsonObj = new JSONObject(jsonString);
                     title=aJsonObj.getString(TAG_HEADLINE);
                     perex=aJsonObj.getString(TAG_PEREX);
                     text=aJsonObj.getString(TAG_TEXT);
                     date=aJsonObj.getString(TAG_DATE);
                    article= new Article(title, idArticle,text,perex,date,"");

                    tvTitle=(TextView)getActivity().findViewById(R.id.aTitle);
                    tvDate=(TextView)getActivity().findViewById(R.id.aDate);
                    webview=(WebView)getActivity().findViewById(R.id.aText);
                    tvTitle.setText(article.title);
                    tvDate.setText(article.date);
                    webview.loadData(article.text, "text/html; charset=UTF-8", "UTF-8");

                } catch (JSONException e) {
                    e.printStackTrace();
                }
            } else {
                Log.e("ServiceHandler", "Couldn't get any data from the url");
            }
        }
    }
}
